//
//
//   config.cc - loop design parameters
//
//   Copyright (C) 2015 by Matt Roberts.
//   License: GNU GPL3 (www.gnu.org)
//
//

#include "config.h"

namespace LoopFormulae {
	//
	//  utility for converting scientific notation
	//
	static double sciNot(double coef, double ex) {
		return coef * ::pow(10.0, ex);
	}

	// constant part of Rr
	const double RrCoef = sciNot(3.38, -8.0);

	//
	//  compute radiation resistance (Rr)
	//
	double Rr(const LoopConfig &c) {
		return RrCoef * pow((c.f * c.f) * A(c), 2.0);
	}

	// constant part of Rloss
	const double RlossCoef = sciNot(9.96, -4.0);

	//
	//  compute loss resistance
	//
	double Rloss(const LoopConfig &c) {
		return RlossCoef * sqrt(c.f) * (S(c) / c.d);
	}

	//
	//  compute efficiency
	//
	double Efficiency(const LoopConfig &c) {
		double rr = Rr(c);
		double rloss = Rloss(c);
		return rr / (rr + rloss);
	}

	// constant part of Rloss
	const double EffCoef1 = sciNot(1.9, -8.0);

	//
	//  compute the inductance (L)
	//
	double L(const LoopConfig &c) {
		return EffCoef1 * S(c) * (7.353 * log10((96 * S(c)) / (M_PI * c.d)) - 6.386);
	}

	//
	//  compute the inductive reactance (XL)
	//
	double XL(const LoopConfig &c) {
		double l = L(c);
		return 2 * M_PI * c.f * l * pow(10.0, 6.0);
	}

	//
	//  compute the tuning capacitor value
	//
	double Ct(const LoopConfig &c) {
		double xl = XL(c);
		return 1.0 / (2.0 * M_PI * c.f * xl * pow(10.0, 6.0));
	}

	//
	//  compute the Q
	//
	double Q(const LoopConfig &c) {
		double xl = XL(c);
		double rr = Rr(c);
		double rl = Rloss(c);
		return xl / (2.0 * (rr + rl));
	}

	//
	//  compute the bandwidth (delta-f)
	//
	double Bandwidth(const LoopConfig &c) {
		double q = Q(c);
		return sciNot(c.f, 6.0) / q;
	}

	// multiplier to convert MHz to Hz
	const double BandwidthNormMHz2HzCoef = sciNot(1.0, 6);

	//
	//  compute the normalized bandwidth (delta-f / f0)
	//
	double BandwidthNorm(const LoopConfig &c) {
		double bw = Bandwidth(c);
		double fm = c.f * BandwidthNormMHz2HzCoef;
		return bw / fm;
	}

	//
	//  compute the distributed capacitance (Cd)
	//
	double Cd(const LoopConfig &c) {
		return 0.82 * S(c);
	}

	//
	//  compute the cap voltage (Vc)
	//
	double Vc(const LoopConfig &c) {
		double xl = XL(c);
		double q = Q(c);
		return sqrt(c.P * xl * q);
	}

	//
	//  compute the loop current (IL)
	//
	double IL(const LoopConfig &c) {
		double q = Q(c);
		double xl = XL(c);
		return sqrt(c.P * q / xl);
	}

	//
	//  compute the loop area
	//
	double A(const LoopConfig &c) {
		return c.AreaFunc(c.w);
	}

	//
	//  circle area computation
	//
	double LoopConfig::Circle(double w) {
		return M_PI * (w / 2.0) * (w / 2.0);
	}

	//
	//  compute the "smallness" of a loop
	//
	double Smallness(const LoopConfig &c) {
		double qwl = 234.0 / c.f;
		double result = 1 - (S(c) / qwl);
		if (result < 0.0) return 0.0;
		if (result > 1.0) return 1.0; // should not be possible
		return result;
	}

	//
	//  compute the "reasonability" of a loop conductor
	//
	double Conductor(const LoopConfig &c) {
		// compute the ratio of the conductor diameter to the loop diameter
		double reasonableRatio = 10.0;
		double ratio = reasonableRatio * (c.d / 12) / c.w;
		ratio = 1 - ratio;
		if (ratio < 0.0) return 0.0;
		if (ratio > 1.0) return 1.0;
		return ratio;
	}

	//
	//  square area computation
	//
	double LoopConfig::Square(double w) {
		return w * w;
	}

	//
	//  compute loop circumference
	//
	double S(const LoopConfig &c) {
		return M_PI * c.w;
	}
}
