//
//
//   evalop.cc - evaluation operator
//
//   Copyright (C) 2015 by Matt Roberts.
//   License: GNU GPL3 (www.gnu.org)
//
//
//   Modified from OpenBeagle example:
//      Open BEAGLE
//      Copyright (C) 2001-2007 by Christian Gagne and Marc Parizeau
//
//

#include <beagle/GA.hpp>
#include <cmath>

#include "evalop.h"
#include "config.h"
#include "fitness.h"

#define BEAGLE_LOOP_NBOBJECTS 24
//#define EVALUATE_CONDUCTOR_FOR_FIRST_TARGET_ONLY

using namespace Beagle;


namespace Basic {
	LoopEvalOp::LoopEvalOp() :
		EvaluationOp("LoopEvalOp")
	{
		// nop
	}


	Fitness::Handle LoopEvalOp::evaluate(Individual& inIndividual, Context& ioContext)
	{
		Beagle_AssertM(inIndividual.size() == 1);
		GA::FloatVector::Handle lFloatVector = castHandleT<GA::FloatVector>(inIndividual[0]);

		double result = 1.0;
		for (unsigned i = 0; i != m_Freq->size(); ++i) {
			// loop config
			LoopFormulae::LoopConfig config;
			config.f = (*m_Freq)[i];
			config.P = (*m_Power);
			config.w = (*lFloatVector)[0]; // first element is diameter
			config.d = (*lFloatVector)[1]; // second element is conductor thickness

			// fitness function
			LoopFormulae::FitnessProduct product;
			LoopFormulae::FitnessElement e;
			// numerator: Efficiency
			e.Coef = 1.0;
			e.Func = LoopFormulae::Efficiency;
			e.Exp = 1.0;
			product.push_back(e);
			// numerator: Normalized Bandwidth
			e.Coef = 1.0;
			e.Func = LoopFormulae::BandwidthNorm;
			e.Exp = 1.0;
			product.push_back(e);
			// numerator: electrical "smallness"
			e.Coef = 1.0;
			e.Func = LoopFormulae::Smallness;
			e.Exp = 1.0;
			product.push_back(e);
			#ifdef EVALUATE_CONDUCTOR_FOR_FIRST_TARGET_ONLY
			if (i == 0) {
			#endif
			// numerator: conductor "reasonableness"
			e.Coef = 1.0;
			e.Func = LoopFormulae::Conductor;
			e.Exp = 1.0;
			product.push_back(e);
			#ifdef EVALUATE_CONDUCTOR_FOR_FIRST_TARGET_ONLY
			}
			#endif
			LoopFormulae::Fitness f(product);
			result *= f.Value(config);
		}

		// compute result
		FitnessSimple::Handle lFitness = new FitnessSimple(result);
		return lFitness;
	}


	void LoopEvalOp::initialize(System& ioSystem)
	{
		Beagle::EvaluationOp::initialize(ioSystem);

		if(ioSystem.getRegister().isRegistered("rf.input.frequency")) {
			m_Freq = castHandleT<Vector>(ioSystem.getRegister()["rf.input.frequency"]);
		} else {
			m_Freq = new Vector(1);
			(*m_Freq) += 7.1;
			Register::Description lDescription(
					"Loop Frequency",
					"Float",
					"7.1",
					"The loop design frequency, in MHz."
					);
			ioSystem.getRegister().addEntry("rf.input.frequency", m_Freq, lDescription);
		}

		if(ioSystem.getRegister().isRegistered("rf.input.power")) {
			m_Power = castHandleT<Float>(ioSystem.getRegister()["rf.input.power"]);
		} else {
			m_Power = new Float(100.0);
			Register::Description lDescription(
					"Loop Power",
					"Float",
					"100",
					"The loop design power, in watts."
					);
			ioSystem.getRegister().addEntry("rf.input.power", m_Power, lDescription);
		}
#if 0
		std::cerr << "DEBUG: using frequency: " << config.f << std::endl;
		std::cerr << "DEBUG: using power    : " << config.P << std::endl;
#endif
	}


	void LoopEvalOp::postInit(System& ioSystem)
	{
		Beagle::EvaluationOp::postInit(ioSystem);
	}
}
// EOF:
